"""Artifact detection for different languages."""

from .python_plot_detection import PYTHON_PLOT_DETECTION_CODE
from .r_plot_detection import R_PLOT_DETECTION_CODE

__all__ = ["PYTHON_PLOT_DETECTION_CODE", "R_PLOT_DETECTION_CODE"]
